﻿using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading;
using System.Windows;
using System.Windows.Input;
using Microsoft.Expression.Interactivity.Core;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Notification;
using NMessenger.WP7ServiceReference;
using NMessenger.AuthenticationServiceReference;

namespace NMessenger.ViewModels
{
    public class MainPageViewModel : INotifyPropertyChanged
    {
        /// <summary>
        /// PropertyChanged Event
        /// </summary>
        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        protected void FirePropertyChanged(string PropertyName)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (null != handler)
            {
                if (Deployment.Current.Dispatcher != null &&
                    !Deployment.Current.Dispatcher.CheckAccess())
                {
                    Deployment.Current.Dispatcher.BeginInvoke
                        (
                            (ThreadStart)(
                            () => handler(this, new PropertyChangedEventArgs(PropertyName))
                            )
                        );
                }
                else
                {
                    handler(this, new PropertyChangedEventArgs(PropertyName));
                }
            }
        }

        #endregion

        //푸쉬 채널생성
        HttpNotificationChannel pushChannel;
        //자신의 채널 uri
        string channelUri;
        public string ChannelUri 
        {
            get { return channelUri; }
            set 
            {
                channelUri = value;
                FirePropertyChanged("ChannelUri");
            }
        }
        //상대방의 채널 uri
        string channelUriTarget;

        //푸쉬 채널이름 설정
        string channelName = "Kaki104NMessengerChannel";

        //쿠키 컨테이너
        CookieContainer cookieContainer = null;
        //Soap Client 
        WP7DomainServiceSoapClient soapClient = null;

        //Authentication Soap Client
        AuthenticationServiceSoapClient authClient;

        ObservableCollection<WP7UriList> wp7Collection;
        /// <summary>
        /// WP7UriList Collection
        /// </summary>
        public ObservableCollection<WP7UriList> WP7Collection 
        {
            get { return wp7Collection; }
            set
            {
                wp7Collection = value;
                FirePropertyChanged("WP7Collection");
            }
        }

        WP7UriList currentData;
        //현재 사용자의 정보
        public WP7UriList CurrentData 
        {
            get { return currentData; }
            set
            {
                currentData = value;
                FirePropertyChanged("CurrentData");
            }
        }

        private string reciveMessage;
        /// <summary>
        /// 수신 메시지
        /// </summary>
        public string ReciveMessage
        {
            get { return reciveMessage; }
            set 
            {
                reciveMessage = value;
                FirePropertyChanged("ReciveMessage");
            }
        }

        private string sendMessage;
        /// <summary>
        /// 송신 메시지
        /// </summary>
        public string SendMessage
        {
            get { return sendMessage; }
            set
            {
                sendMessage = value;
                FirePropertyChanged("SendMessage");
            }
        }

        string notificationStatus;
        /// <summary>
        /// 노티피케이션 상태 Received 메시지 받음, QueueFull 큐풀(재전송), Suppressed(취소), N/A(에러), Dropped(드랍됨)
        /// </summary>
        public string NotificationStatus
        {
            get { return notificationStatus; }
            set
            {
                notificationStatus = value;
                FirePropertyChanged("NotificationStatus");
            }
        }

        string notificationChannelStatus;
        /// <summary>
        /// 노티피케이션 채널 상태 Connected, Temporarily Disconnected, N/A
        /// </summary>
        public string NotificationChannelStatus
        {
            get { return notificationChannelStatus; }
            set
            {
                notificationChannelStatus = value;
                FirePropertyChanged("NotificationChannelStatus");
            }
        }

        string deviceConnectionStatus ;
        /// <summary>
        /// 디바이스 연결 상태
        /// </summary>
        public string DeviceConnectionStatus
        {
            get { return deviceConnectionStatus; }
            set
            {
                deviceConnectionStatus = value;
                FirePropertyChanged("DeviceConnectionStatus");
            }
        }

        private string userID;
        /// <summary>
        /// 사용자 아이디
        /// </summary>
        public string UserID
        {
            get { return userID; }
            set
            {
                userID = value;
                FirePropertyChanged("UserID");
            }
        }

        private string userPW;
        /// <summary>
        /// 사용자 비밀번호
        /// </summary>
        public string UserPW
        {
            get { return userPW; }
            set
            {
                userPW = value;
                FirePropertyChanged("UserPW");
            }
        }

        private string userName;
        /// <summary>
        /// 사용자 이름
        /// </summary>
        public string UserName
        {
            get { return userName; }
            set
            {
                userName = value;
                FirePropertyChanged("UserName");
            }
        }

        private bool isLogin;
        /// <summary>
        /// 로그인 여부
        /// </summary>
        public bool IsLogin
        {
            get { return isLogin; }
            set
            {
                isLogin = value;
                FirePropertyChanged("IsLogin");
            }
        }

        private string messageData;
        /// <summary>
        /// 메세지 데이터
        /// </summary>
        public string MessageData
        {
            get { return messageData; }
            set
            {
                if (messageData != value && value != null)
                {
                    messageData = value;
                    FirePropertyChanged("MessageData");
                }
            }
        }

        /// <summary>
        /// 생성자
        /// </summary>
        public MainPageViewModel()
        {
            //도메인 서비스 솝 클라이언트 
            soapClient = new WP7DomainServiceSoapClient();
            //조회 완료 이벤트 처리 
            soapClient.GetWP7UriListSetCompleted += new EventHandler<GetWP7UriListSetCompletedEventArgs>(soapClient_GetWP7UriListSetCompleted);
            //저장 완료 이벤트 처리
            soapClient.SubmitChangesCompleted += new EventHandler<NMessenger.WP7ServiceReference.SubmitChangesCompletedEventArgs>(soapClient_SubmitChangesCompleted);

            //인증 서비스 솝 클라이언트
            authClient = new AuthenticationServiceSoapClient();
            authClient.LoginCompleted += new EventHandler<LoginCompletedEventArgs>(authClient_LoginCompleted);
            authClient.LogoutCompleted += new EventHandler<LogoutCompletedEventArgs>(authClient_LogoutCompleted);
        }

        /// <summary>
        /// 로그아웃 완료
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void authClient_LogoutCompleted(object sender, LogoutCompletedEventArgs e)
        {
            if (e.Error == null)
            {
                var logout = e.Result.LogoutResult.RootResults.FirstOrDefault().DisplayName;
                //string item = "Logout User Name : " + logout + " Login Time :" + DateTime.Now;
                UserName = "";
                IsLogin = false;
                MessageData = "You have been logged out.";
            }
        }

        /// <summary>
        /// 로그인 완료
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void authClient_LoginCompleted(object sender, LoginCompletedEventArgs e)
        {
            if (e.Error == null && e.Result.LoginResult.RootResults != null)
            {
                UserName = e.Result.LoginResult.RootResults.FirstOrDefault().DisplayName;
                IsLogin = true;
                MessageData = "Login was successful.";
            }
            else
            {
                MessageData = "Can not login.";
            }
        }

        /// <summary>
        /// 저장 완료 이벤트 처리
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void soapClient_SubmitChangesCompleted(object sender, NMessenger.WP7ServiceReference.SubmitChangesCompletedEventArgs e)
        {
            if (e.Error == null)
            {
                //저장 완료 후 반환된 결과를 받아서
                ObservableCollection<NMessenger.WP7ServiceReference.ChangeSetEntry> returnChangeSet = e.Result.SubmitChangesResult;
                //그중 처음 데이터의 오퍼레이션을 확인하고
                var operation = returnChangeSet.FirstOrDefault().Operation;
                //오퍼레이션 종류에 따라 메시지 출력
                switch (operation)
                {
                    case NMessenger.WP7ServiceReference.DomainOperation.Insert:
                        MessageData = "My channel uri regist successfully";
                        CurrentData = returnChangeSet.FirstOrDefault().Entity as WP7UriList;
                        break;
                    case NMessenger.WP7ServiceReference.DomainOperation.Update:
                        MessageData = "My channel uri modify successfully";
                        break;
                    case NMessenger.WP7ServiceReference.DomainOperation.Delete:
                        MessageData = "My channel uri unregist successfully";
                        CurrentData = null;
                        break;
                }
                if (1 == 1)
                {
                }
            }
            else
            {
                MessageData = "The operation failed.[" + e.Error.Message + "]";
            }
        }
        //조회 완료시 컬렉션 설정
        void soapClient_GetWP7UriListSetCompleted(object sender, GetWP7UriListSetCompletedEventArgs e)
        {
            WP7Collection = new ObservableCollection<WP7UriList>(e.Result.GetWP7UriListSetResult.RootResults);
        }

        private ICommand addCommand;
        /// <summary>
        /// 추가 커맨드
        /// </summary>
        public ICommand AddCommand
        {
            get
            {
                if (addCommand == null)
                {
                    addCommand = new ActionCommand(InputUserName =>
                    {
                        //입력 받은 사용자 이름을 확인
                        string userName = InputUserName as string;
                        if (userName != null && userName != "")
                        {
                            //폰의 ChannelUri 확인
                            if (ChannelUri == null && ChannelUri == "")
                            {
                                MessageData = "no chann uri, try again";
                                return;
                            }
                            //새로운 레코드 생성
                            WP7UriList addItem = new WP7UriList 
                            {
                                UserName = userName,
                                ChannelUri = ChannelUri,
                                RegDT = DateTime.Now,
                                UptDT = DateTime.Now,
                                UseBT = true
                            };

                            //체인지셋 구성
                            ObservableCollection<NMessenger.WP7ServiceReference.ChangeSetEntry> addedItems = new ObservableCollection<NMessenger.WP7ServiceReference.ChangeSetEntry>();
                            NMessenger.WP7ServiceReference.ChangeSetEntry addChangeSetEntry = new NMessenger.WP7ServiceReference.ChangeSetEntry 
                            {
                                OriginalEntity = null,
                                Entity = addItem,
                                Operation = NMessenger.WP7ServiceReference.DomainOperation.Insert
                            };
                            //체인지셋에 새로만든 레코드 추가
                            addedItems.Add(addChangeSetEntry);
                            //SubmitChangeRequest 생성
                            NMessenger.WP7ServiceReference.SubmitChangesRequest request = new NMessenger.WP7ServiceReference.SubmitChangesRequest(addedItems);
                            //SubmitChangeAsync 실행
                            soapClient.SubmitChangesAsync(request, this);
                        }
                    }
                    );
                }
                return addCommand;
            }
        }
        private ICommand cancelCommand;
        /// <summary>
        /// 취소 커맨드
        /// </summary>
        public ICommand CancelCommand
        {
            get
            {
                if (cancelCommand == null)
                {
                    cancelCommand = new ActionCommand(() =>
                    {
                        MessageData = "지원하지 않는 명령입니다.";
                    });
                }
                return cancelCommand;
            }
        }

        private ICommand removeCommand;
        /// <summary>
        /// 삭제 커맨드 - 현재 사용자의 정보를 서버에서 완전히 삭제 한다
        /// </summary>
        public ICommand RemoveCommand
        {
            get
            {
                if (removeCommand == null)
                {
                    removeCommand = new ActionCommand(() =>
                    {
                        if (CurrentData != null)
                        {
                            //체인지셋 구성
                            ObservableCollection<NMessenger.WP7ServiceReference.ChangeSetEntry> removeItems = new ObservableCollection<NMessenger.WP7ServiceReference.ChangeSetEntry>();
                            NMessenger.WP7ServiceReference.ChangeSetEntry removeChangeSetEntry = new NMessenger.WP7ServiceReference.ChangeSetEntry
                            {
                                OriginalEntity = CurrentData,
                                Entity = CurrentData,
                                Operation = NMessenger.WP7ServiceReference.DomainOperation.Delete
                            };
                            //체인지셋에 수정할 데이터 입력
                            removeItems.Add(removeChangeSetEntry);
                            //SubmitChangeRequest 생성
                            NMessenger.WP7ServiceReference.SubmitChangesRequest request = new NMessenger.WP7ServiceReference.SubmitChangesRequest(removeItems);

                            //인증 쿠키 입력
                            soapClient.CookieContainer = authClient.CookieContainer;
                            //SubmitChangeAsync 실행
                            soapClient.SubmitChangesAsync(request, this);
                        }
                    });
                }
                return removeCommand;
            }
        }

        private ICommand saveCommand;
        /// <summary>
        /// 저장 커맨드 - 현재 사용자의 정보를 사용하지 못하도록 업데이트 한다.
        /// </summary>
        public ICommand SaveCommand
        {
            get
            {
                if (saveCommand == null)
                {
                    saveCommand = new ActionCommand(() =>
                    {
                        if (CurrentData != null)
                        {
                            CurrentData.UseBT = false;

                            //체인지셋 구성
                            ObservableCollection<NMessenger.WP7ServiceReference.ChangeSetEntry> updateItems = new ObservableCollection<NMessenger.WP7ServiceReference.ChangeSetEntry>();
                            NMessenger.WP7ServiceReference.ChangeSetEntry updateChangeSetEntry = new NMessenger.WP7ServiceReference.ChangeSetEntry
                            {
                                OriginalEntity = CurrentData,
                                Entity = CurrentData,
                                Operation = NMessenger.WP7ServiceReference.DomainOperation.Update
                            };
                            //체인지셋에 수정할 데이터 입력
                            updateItems.Add(updateChangeSetEntry);
                            //SubmitChangeRequest 생성
                            NMessenger.WP7ServiceReference.SubmitChangesRequest request = new NMessenger.WP7ServiceReference.SubmitChangesRequest(updateItems);
                            //SubmitChangeAsync 실행
                            soapClient.SubmitChangesAsync(request, this);
                        }
                    });
                }
                return saveCommand;
            }
        }

        private ICommand selectCommand;
        /// <summary>
        /// 조회 커맨드 
        /// </summary>
        public ICommand SelectCommand
        {
            get
            {
                if (selectCommand == null)
                {
                    selectCommand = new ActionCommand(() =>
                    {
                        //조회 함수 호출
                        GetWP7UriListSetRequest request = new GetWP7UriListSetRequest();
                        soapClient.GetWP7UriListSetAsync(request, this);
                    });
                }
                return selectCommand;
            }
        }

        private ICommand loadedCommand;
        public ICommand LoadedCommand
        {
            get
            {
                if (loadedCommand == null)
                {
                    loadedCommand = new ActionCommand(() =>
                    {
                        //채널이름 찾아보기
                        pushChannel = HttpNotificationChannel.Find(channelName);
                        if (pushChannel == null)
                        {
                            //생성된 채널이 없으면 생성
                            pushChannel = new HttpNotificationChannel(channelName);
                            //채널정보 업데이트시 이벤트 연결
                            pushChannel.ChannelUriUpdated += new EventHandler<NotificationChannelUriEventArgs>(pushChannel_ChannelUriUpdated);
                            //채널 오류 발생 이벤트 연결
                            pushChannel.ErrorOccurred += new EventHandler<NotificationChannelErrorEventArgs>(pushChannel_ErrorOccurred);
                            //채널 열기
                            pushChannel.Open();
                        }
                        else
                        {
                            //채널 업데이트 이벤트 연결
                            pushChannel.ChannelUriUpdated += new EventHandler<NotificationChannelUriEventArgs>(pushChannel_ChannelUriUpdated);
                            //채널 오류 발생 이벤트 연결
                            pushChannel.ErrorOccurred += new EventHandler<NotificationChannelErrorEventArgs>(pushChannel_ErrorOccurred);
                            //채널 Uri정보 조회
                            //System.Diagnostics.Debug.WriteLine("[ChannelUri] " + pushChannel.ChannelUri.ToString());

                            //채널 정보를 미리 저장
                            ChannelUri = pushChannel.ChannelUri.ToString();
                        }
                    });
                }
                return loadedCommand;
            }
        }

        private ICommand sendMessageCommand;
        /// <summary>
        /// 메세지 전송 커맨드 
        /// </summary>
        public ICommand SendMessageCommand
        {
            get
            {
                if (sendMessageCommand == null)
                {
                    sendMessageCommand = new ActionCommand(() =>
                    {
                        //if(channelUriTarget != null && channelUriTarget != "")
                        //    SendNMessenger();
                        SendNMessenger();
                    });
                }
                return sendMessageCommand;
            }
        }

        private ICommand selectionChangedCommand;
        /// <summary>
        /// 셀렉션 체인지드 커맨드
        /// </summary>
        public ICommand SelectionChangedCommand
        {
            get
            {
                if (selectionChangedCommand == null)
                {
                    selectionChangedCommand = new ActionCommand(ctl =>
                    {
                        WP7UriList uri = (ctl as ListPicker).SelectedItem as WP7UriList;
                        if (uri != null)
                        {
                            channelUriTarget = uri.ChannelUri;
                            if (channelUriTarget.Length != 120)
                            { 
                            }
                            //System.Diagnostics.Debug.WriteLine("ChannelUri       : " + ChannelUri);
                            //System.Diagnostics.Debug.WriteLine("channelUriTarget : " + channelUriTarget);
                        }
                    });
                }
                return selectionChangedCommand;
            }
        }

        private ICommand loginCommand;
        /// <summary>
        /// 로그인 커맨드
        /// </summary>
        public ICommand LoginCommand
        {
            get
            {
                if (loginCommand == null)
                {
                    loginCommand = new ActionCommand(() =>
                    {
                        var login = new LoginRequest(UserID, UserPW, false, null);
                        authClient.LoginAsync(login);
                    });
                }
                return loginCommand;
            }
        }

        private ICommand logoutCommand;
        /// <summary>
        /// 로그아웃 커맨드
        /// </summary>
        public ICommand LogoutCommand
        {
            get
            {
                if (logoutCommand == null)
                {
                    logoutCommand = new ActionCommand(() =>
                    {
                        var logout = new LogoutRequest();
                        authClient.LogoutAsync(logout);
                    });
                }
                return logoutCommand;
            }
        }

        void pushChannel_ChannelUriUpdated(object sender, NotificationChannelUriEventArgs e)
        {
            //sender를 푸시체널로 형변환하고
            HttpNotificationChannel pushChannel = sender as HttpNotificationChannel;
            //푸시체널을 어플리케이션타일과 바인딩
            //pushChannel.BindToShellTile();

            //이넘이 raw notification 인듯
            pushChannel.HttpNotificationReceived += new EventHandler<HttpNotificationEventArgs>(pushChannel_HttpNotificationReceived);

            //변경된 채널 Uri정보 디버그 창에 출력
            //System.Diagnostics.Debug.WriteLine("[ChannelUri] " + e.ChannelUri.ToString());

            //채널 정보를 미리 저장
            ChannelUri = pushChannel.ChannelUri.ToString();
        }

        //Raw Notification 처리
        void pushChannel_HttpNotificationReceived(object sender, HttpNotificationEventArgs e)
        {
            if (e.Notification.Body != null && e.Notification.Headers != null)
            {
                StreamReader reader = new StreamReader(e.Notification.Body);
                ReciveMessage = reader.ReadToEnd();
            }
        }

        //Error
        void pushChannel_ErrorOccurred(object sender, NotificationChannelErrorEventArgs e)
        {
            //발생된 에러 메시지 조회
            MessageBox.Show(String.Format("푸쉬 노티피케이션 {0} 에러 발생.  {1} ({2}) {3}",
                                e.ErrorType, e.Message, e.ErrorCode, e.ErrorAdditionalData)
                                );
        }

        //Self Raw Notification Send
        private void SendNMessenger()
        {
            Uri sendUri = new Uri(channelUriTarget, UriKind.Absolute);
            //Uri sendUri = new Uri(ChannelUri, UriKind.Absolute);

            //Request 생성
            HttpWebRequest request = (HttpWebRequest)WebRequest.Create(sendUri);
            //POST로 지정
            request.Method = "POST";
            //뒤에 utf-8을 넣어 주어야 한글도 깨지지 않고 넘어감
            request.ContentType = "text/xml;charset=utf-8";
            //Raw Notification
            request.Headers["X-NotificationClass"] = "3";

            request.BeginGetRequestStream(new AsyncCallback(channelRequest), request);
        }

        //보내는 곳과 연결 하고
        private void channelRequest(IAsyncResult asyncResult)
        {
            HttpWebRequest request = (HttpWebRequest)asyncResult.AsyncState;
            //몸통 만들고
            Stream body = request.EndGetRequestStream(asyncResult);
            //전송할 데이터 만들고
            byte[] contents = Encoding.UTF8.GetBytes(SendMessage);
            //전송
            body.Write(contents, 0, contents.Length);
            body.Flush();
            body.Close();

            request.BeginGetResponse(new AsyncCallback(channelResponse), request);
            //Deployment.Current.Dispatcher.BeginInvoke(() => 
            //{
            //});
        }

        string allStatus;
        public string AllStatus 
        {
            get { return allStatus; }
            set 
            {
                allStatus = value;
                FirePropertyChanged("AllStatus");
            }
        }

        private void channelResponse(IAsyncResult asyncResult)
        {
            var request = (HttpWebRequest)asyncResult.AsyncState;
            var response = request.EndGetResponse(asyncResult);
            //전송 결과 뽑아내고
            NotificationStatus = response.Headers["X-NotificationStatus"];
            NotificationChannelStatus = response.Headers["X-SubscriptionStatus"];
            DeviceConnectionStatus = response.Headers["X-DeviceConnectionStatus"];
            AllStatus = NotificationStatus
                        + " " + NotificationChannelStatus
                        + " " + DeviceConnectionStatus;
        }

    }
}
